@@ -5,10 +5,18 @@
 #include <stdio.h>
 #include "kiss_fft.h"
 
+#define CELT_C 
+#include "../libcelt/stack_alloc.h"
+#include "../libcelt/kiss_fft.c"
+
+
 #ifndef M_PI
 #define M_PI 3.141592653
 #endif
 
+#ifdef FIXED_DEBUG  
+long long celt_mips=0;
+#endif
 int ret = 0;
 
 void check(kiss_fft_cpx  * in,kiss_fft_cpx  * out,int nfft,int isinverse)
@@ -99,6 +107,7 @@ void test1d(int nfft,int isinverse)
 
 int main(int argc,char ** argv)
 {
+    ALLOC_STACK;
     if (argc>1) {
         int k;
         for (k=1;k<argc;++k) {
