@@ -4,12 +4,17 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include "laplace.c"
-#include "stack_alloc.h"
-#include "entenc.c"
-#include "entdec.c"
-#include "rangeenc.c"
-#include "rangedec.c"
+#include "laplace.h"
+#define CELT_C 
+#include "../libcelt/stack_alloc.h"
+
+#include "../libcelt/rangeenc.c"
+#include "../libcelt/rangedec.c"
+#include "../libcelt/entenc.c"
+#include "../libcelt/entdec.c"
+#include "../libcelt/entcode.c"
+#include "../libcelt/laplace.c"
+
 
 int main(void)
 {
