@@ -1,4 +1,5 @@
 /* Copyright (C) 2002-2008 Jean-Marc Valin 
+   Copyright (C) 2008-2009 Gregory Maxwell
    File: celtenc.c
 
    Redistribution and use in source and binary forms, with or without
@@ -79,12 +80,14 @@ int oe_write_page(ogg_page *page, FILE *fp)
 }
 
 #define MAX_FRAME_SIZE 2000
-#define MAX_FRAME_BYTES 2000
+#define MAX_FRAME_BYTES 300
+#define IMIN(a,b) ((a) < (b) ? (a) : (b))   /**< Minimum int value.   */
+#define IMAX(a,b) ((a) > (b) ? (a) : (b))   /**< Maximum int value.   */
 
 /* Convert input audio bits, endians and channels */
 static int read_samples(FILE *fin,int frame_size, int bits, int channels, int lsb, short * input, char *buff, celt_int32_t *size)
 {   
-   unsigned char in[MAX_FRAME_BYTES*2];
+   unsigned char in[MAX_FRAME_SIZE*2];
    int i;
    short *s;
    int nb_read;
@@ -184,13 +187,13 @@ void add_fisbone_packet (ogg_stream_state *os, celt_int32_t serialno, CELTHeader
 
 void version(void)
 {
-   printf ("celtenc (CELT encoder)\n");
+   printf ("celtenc (CELT %s encoder)\n",CELT_VERSION);
    printf ("Copyright (C) 2008 Jean-Marc Valin\n");
 }
 
 void version_short(void)
 {
-   printf ("celtenc (CELT encoder)\n");
+   printf ("celtenc (CELT %s encoder)\n",CELT_VERSION);
    printf ("Copyright (C) 2008 Jean-Marc Valin\n");
 }
 
@@ -210,8 +213,12 @@ void usage(void)
    printf ("  -                 stdout\n");
    printf ("\n");  
    printf ("Options:\n");
-   printf (" --bitrate n        Encoding bit-rate\n"); 
+   printf (" --bitrate n        Encoding bit-rate in kbit/sec\n"); 
+   printf (" --vbr              Use variable bitrate encoding\n"); 
    printf (" --comp n           Encoding complexity (0-10)\n");
+   printf (" --framesize n      Frame size (Default: 256)\n");
+   printf (" --noltp            Do not use long-term prediction\n");
+   printf (" --independent      Encode frames independently (implies noltp)\n");
    printf (" --skeleton         Outputs ogg skeleton metadata (may cause incompatibilities)\n");
    printf (" --comment          Add the given string as an extra comment. This may be\n");
    printf ("                     used multiple times\n");
@@ -240,17 +247,24 @@ int main(int argc, char **argv)
    char *inFile, *outFile;
    FILE *fin, *fout;
    short input[MAX_FRAME_SIZE];
-   celt_int32_t frame_size;
+   celt_int32_t frame_size = 256;
    int quiet=0;
    int nbBytes;
    CELTMode *mode;
    void *st;
    unsigned char bits[MAX_FRAME_BYTES];
+   int with_vbr = 0;
    int with_skeleton = 0;
+   int total_bytes = 0;
+   int peak_bytes = 0;
    struct option long_options[] =
    {
       {"bitrate", required_argument, NULL, 0},
+      {"vbr",no_argument,NULL, 0},
       {"comp", required_argument, NULL, 0},
+      {"noltp", no_argument, NULL, 0},
+      {"independent", no_argument, NULL, 0},
+      {"framesize", required_argument, NULL, 0},
       {"skeleton",no_argument,NULL, 0},
       {"help", no_argument, NULL, 0},
       {"quiet", no_argument, NULL, 0},
@@ -286,16 +300,15 @@ int main(int argc, char **argv)
    int comments_length;
    int close_in=0, close_out=0;
    int eos=0;
-   celt_int32_t bitrate=-1;
+   float bitrate=-1;
    char first_bytes[12];
    int wave_input=0;
    celt_int32_t lookahead = 0;
    int bytes_per_packet=48;
    int complexity=-127;
-   
-   snprintf(vendor_string, sizeof(vendor_string), "Encoded with CELT\n");
-   
-   comment_init(&comments, &comments_length, vendor_string);
+   int prediction=2; 
+   int bitstream;
+
 
    /*Process command-line options*/
    while(1)
@@ -310,7 +323,10 @@ int main(int argc, char **argv)
       case 0:
          if (strcmp(long_options[option_index].name,"bitrate")==0)
          {
-            bitrate = atoi (optarg);
+            bitrate = atof (optarg);
+         } else if (strcmp(long_options[option_index].name,"vbr")==0)
+         {
+            with_vbr=1;
          } else if (strcmp(long_options[option_index].name,"skeleton")==0)
          {
             with_skeleton=1;
@@ -353,6 +369,16 @@ int main(int argc, char **argv)
          } else if (strcmp(long_options[option_index].name,"comp")==0)
          {
             complexity=atoi (optarg);
+         } else if (strcmp(long_options[option_index].name,"framesize")==0)
+         {
+            frame_size=atoi (optarg);
+         } else if (strcmp(long_options[option_index].name,"noltp")==0)
+         {
+            if (prediction>1)
+              prediction=1;
+         } else if (strcmp(long_options[option_index].name,"independent")==0)
+         {
+              prediction=0;
          } else if (strcmp(long_options[option_index].name,"comment")==0)
          {
 	   if (!strchr(optarg, '='))
@@ -440,34 +466,40 @@ int main(int argc, char **argv)
       }
    }
 
-   if (chan == 1)
-   {
-      if (bitrate < 0)
-         bitrate = 64;
-      if (bitrate < 32)
-         bitrate = 32;
-      if (bitrate > 110)
-         bitrate = 110;
-   }
-   else if (chan == 2)
-   {
-      if (bitrate < 0)
-         bitrate = 128;
-      if (bitrate < 64)
-         bitrate = 64;
-      if (bitrate > 150)
-         bitrate = 150;
-   } else {
-      fprintf (stderr, "Only mono and stereo are supported\n");
-      return 1;
+   if (bitrate<=0.005)
+     if (chan==1)
+       bitrate=64.0;
+     else
+       bitrate=128.0;
+     
+   bytes_per_packet = (bitrate*1000*frame_size/rate+4)/8;
+   
+   if (bytes_per_packet < 8) {
+      bytes_per_packet=8;
+      fprintf (stderr, "Warning: Requested bitrate (%0.3fkbit/sec) is too low. Setting CELT to 8 bytes/frame.\n",bitrate);
+   } else if (bytes_per_packet > MAX_FRAME_BYTES) {
+      bytes_per_packet=MAX_FRAME_BYTES;
+      fprintf (stderr, "Warning: Requested bitrate (%0.3fkbit/sec) is too high. Setting CELT to %d bytes/frame.\n",bitrate,MAX_FRAME_BYTES);      
    }
 
-   mode = celt_mode_create(rate, chan, 256, NULL);
+   if (with_vbr)
+   {
+     /*In VBR mode the bytes_per_packet argument becomes a hard maximum. 3x the average rate is just a random choice.*/
+     bytes_per_packet=IMIN(bytes_per_packet*3,MAX_FRAME_BYTES);
+   } else { 
+     bitrate = ((rate/(float)frame_size)*8*bytes_per_packet)/1000.0;
+   }
+   
+   mode = celt_mode_create(rate, chan, frame_size, NULL);
    if (!mode)
       return 1;
-   celt_mode_info(mode, CELT_GET_FRAME_SIZE, &frame_size);
-   
-   bytes_per_packet = (bitrate*1000*frame_size/rate+4)/8;
+
+  celt_mode_info(mode,CELT_GET_BITSTREAM_VERSION,&bitstream);      
+
+   snprintf(vendor_string, sizeof(vendor_string), "Encoded with CELT %s (bitstream: %d)\n",CELT_VERSION,bitstream);
+   comment_init(&comments, &comments_length, vendor_string);
+
+   celt_mode_info(mode, CELT_GET_FRAME_SIZE, &frame_size);   
    
    celt_header_init(&header, mode);
    header.nb_channels = chan;
@@ -477,15 +509,33 @@ int main(int argc, char **argv)
       if (chan==2)
          st_string="stereo";
       if (!quiet)
-         fprintf (stderr, "Encoding %d Hz audio using %s (%d bytes per packet)\n", 
-               header.sample_rate, st_string, bytes_per_packet);
+         if (with_vbr)
+           fprintf (stderr, "Encoding %d Hz %s audio in %d sample packets at %0.3fkbit/sec (%d maximum bytes per packet) with bitstream version %d\n", 
+               header.sample_rate, st_string, frame_size, bitrate, bytes_per_packet,bitstream);
+         else      
+           fprintf (stderr, "Encoding %d Hz %s audio in %d sample packets at %0.3fkbit/sec (%d bytes per packet) with bitstream version %d\n", 
+               header.sample_rate, st_string, frame_size, bitrate, bytes_per_packet,bitstream);
    }
-   /*fprintf (stderr, "Encoding %d Hz audio at %d bps using %s mode\n", 
-     header.rate, mode->bitrate, mode->modeName);*/
 
    /*Initialize CELT encoder*/
    st = celt_encoder_create(mode);
 
+   if (with_vbr)
+   {
+     int tmp = (bitrate*1000);
+     if (celt_encoder_ctl(st, CELT_SET_VBR_RATE(tmp)) != CELT_OK)
+     {
+        fprintf (stderr, "VBR request failed\n");
+        return 1;
+     }
+   }     
+
+   if (celt_encoder_ctl(st, CELT_SET_PREDICTION(prediction)) != CELT_OK)
+   {
+      fprintf (stderr, "Prediction request failed\n");
+      return 1;
+   }
+
    if (complexity!=-127) {
      if (celt_encoder_ctl(st, CELT_SET_COMPLEXITY(complexity)) != CELT_OK)
      {
@@ -620,6 +670,8 @@ int main(int argc, char **argv)
          break;
       }
       nb_encoded += frame_size;
+      total_bytes += nbBytes;
+      peak_bytes=IMAX(nbBytes,peak_bytes);
 
       if (wave_input)
       {
@@ -678,6 +730,9 @@ int main(int argc, char **argv)
          bytes_written += ret;
    }
 
+   if (with_vbr && !quiet)
+     fprintf (stderr, "Average rate %0.3fkbit/sec, %d peak bytes per packet\n", (total_bytes*8.0/((float)nb_encoded/header.sample_rate))/1000.0, peak_bytes);
+
    celt_encoder_destroy(st);
    celt_mode_destroy(mode);
    ogg_stream_clear(&os);
